# -*- coding: utf-8 -*-
import re
from copy import copy

from django.contrib import admin
from django.contrib.admin.views.main import ChangeList

import django_filters

class AdvancedSearchModelAdmin(admin.ModelAdmin):

    def get_filterset_class(self, request):
        """Return a filterset_class suitable for advanced search
        first look if attribute ``advanced_search_fields``
        """
        fields = self.get_advanced_search_fields(request) 
        
        if fields:
            meta = type('Meta', (object,), {'fields': fields, 'model' : self.model})
            AdminFilterset = type('Admin%sFilterset' % self.model.__name__, 
                                  (django_filters.FilterSet,),
                                  { 'Meta' : meta, 'fields' : fields})
            return AdminFilterset
        # elif hasattr(self, 'get_filterset_class'):
        #     return self.get_filterset()
        return None


    def get_advanced_search_fields(self, request):
        """
        Return the list of fields that should be used in advanced search

        :arg request: the request
        """
        return getattr(self, 'advanced_search_fields', None)

    def lookup_allowed(self, lookup, value):
        if hasattr(self, '_lookup_search'):
            if lookup in self._lookup_names:
                return True
        return super(AdvancedSearchModelAdmin, self).lookup_allowed(lookup, value)

    def get_changelist(self, request):
        return AdvancedSearchChangeList

    def queryset(self, request):

        if not hasattr(self, '_search_filterset_class'):
            self._search_filterset_class = self.get_filterset_class(request)

        if self._search_filterset_class:
            self.search_filterset = self._search_filterset_class(request.GET)
            self.search_form = self.search_filterset.form
            self.search_form.is_valid()

            self._lookup_names = re.findall('name="(?P<name>[^\"]*)"', str(self.search_form))

            qs = self.search_filterset.qs
            # ordering is the only remainig action taken in ModelAdmin.queryset...
            ordering = self.get_ordering(request)
            if ordering:
                qs = qs.order_by(*ordering)
            return qs
        return admin.ModelAdmin.queryset(self, request)
            
class AdvancedSearchChangeList(ChangeList):
    """
    A ChangeList that can hadle any filter condition
    added by the search_filterset's form.

    Django uses ModelAdmin's queryset method to initialize 
    ChangeList's root_query_set::

       self.root_query_set = model_admin.queryset(request)

    and at the end of the :meth:`__init__` it handles all filtering
    due according to filter and the like::

       self.query_set = self.get_query_set(request)

   that uses it as base the root_query_set::

       qs = self.root_query_set
       

    
    """
    def get_queryset(self, request):
        # save a copy of the GET parameters 
        saved_params = copy(self.params)
        self.params.pop("_popup", None)
        self.params.pop("_json", None)
        ## ModelAdmin.queryset has already filtered the qs
        #  we now just need to hide the paramenters that belongs to
        #  the filterset_class.form. Standard ChangeList would not
        #  know how to handle them
        if hasattr(self.model_admin, 'search_form'):
            search_form = self.model_admin.search_form

            if search_form.is_valid():
                for name in self.model_admin._lookup_names:
                    self.params.pop(name, None)

        qs_result = ChangeList.get_query_set(self, request)

        # restore the self.params list so that templatetag 'result_list'
        # can compose it correctly. In particular ordering and filters
        # need to know all parameters
        self.params = saved_params

        return qs_result

    get_query_set = get_queryset
